@@ -15,9 +15,9 @@
 fgasp <- function(input, output, have_noise=TRUE, kernel_type='matern_5_2'){
 
 
-  if( (kernel_type!='matern_5_2')&(kernel_type!='exp') ){
+  if( (kernel_type!='matern_5_2')&(kernel_type!='matern_3_2')&(kernel_type!='exp')){
     stop("the current version only support the Matern covariance with smoothness 
-         parameter being 2.5 or 0.5 (exponential kernel). \n")
+         parameter being 2.5, 1.5 or 0.5 (exponential kernel). \n")
   }
   
   ##if the input is not sorted so we will sort them
