@@ -18,7 +18,6 @@ using namespace Eigen;
  
  
 
-//July 16, 2018
 ////Construct_W0_matern_5_2 
 // [[Rcpp::export]] 
 MatrixXd Construct_W0_matern_5_2(const double sigma2, const double lambda){ 
@@ -35,7 +34,22 @@ MatrixXd Construct_W0_matern_5_2(const double sigma2, const double lambda){
   return W0; 
 } 
 
-//July 16, 2018
+
+////Construct_W0_matern_3_2 
+// [[Rcpp::export]] 
+MatrixXd Construct_W0_matern_3_2(const double sigma2, const double lambda){ 
+  //int num_dim=sigma2.size(); 
+  
+  Eigen::MatrixXd W0= Eigen::MatrixXd::Zero(2,2); 
+  //Eigen::MatrixXd d= Eigen::MatrixXd::Zero(3,3); //the first row has all zeros  
+  
+  W0(0,0)=sigma2; 
+  W0(1,1)=sigma2*pow(lambda,2.0); 
+
+  return W0; 
+} 
+
+
 ////Construct_W0_exp 
 // [[Rcpp::export]] 
 MatrixXd Construct_W0_exp(const double sigma2, const double lambda){ 
@@ -49,7 +63,7 @@ MatrixXd Construct_W0_exp(const double sigma2, const double lambda){
 } 
 
 
- 
+
  ////Construct_G_matern_5_2 
  // [[Rcpp::export]] 
  List Construct_G_matern_5_2(Eigen::VectorXd delta_x, double lambda){  //be careful about delta_x,lambda if one only wants to sample one 
@@ -82,9 +96,33 @@ MatrixXd Construct_W0_exp(const double sigma2, const double lambda){
    return GG; 
 } 
 
+////Construct_G_matern_3_2 
+// [[Rcpp::export]] 
+List Construct_G_matern_3_2(Eigen::VectorXd delta_x, double lambda){  
+  int num_obs=delta_x.size()+1; 
+  //int num_dim=lambda.size();  
+  List GG(num_obs);  
+  GG[0]=Eigen::MatrixXd::Zero(2,2); 
+  
+  Eigen::MatrixXd d= Eigen::MatrixXd::Zero(2,2); //the first row has all zeros  
+  double  lambda_delta_x;
+  
+  // num_dim list, each is 2(num_obs)\times 2 list 
+  for(int j_GG=0;j_GG<(num_obs-1);j_GG++){ 
+    int j_GG_1=j_GG+1;
+    lambda_delta_x=lambda*delta_x[j_GG];
+    d(0,0)=1+lambda_delta_x; 
+    d(0,1)=delta_x[j_GG]; 
+    d(1,0)=-pow(lambda,2.0)*delta_x[j_GG];
+    d(1,1)=1-lambda_delta_x; 
+    d=exp(-lambda_delta_x)*d;
+    GG[j_GG_1]=d; 
+  } 
+  return GG; 
+} 
+
+
 ////Construct_G_exp
-//' @keywords internal
-//' @noRd
 // [[Rcpp::export]] 
 List Construct_G_exp(Eigen::VectorXd delta_x, double lambda){  //be careful about delta_x,lambda if one only wants to sample one 
   int num_obs=delta_x.size()+1; 
@@ -120,7 +158,7 @@ List Construct_W_matern_5_2(double sigma2,Eigen::VectorXd delta_x, double lambda
     for(int j_Wi=0;j_Wi<(num_obs-1);j_Wi++){ 
       j_Wi_1= j_Wi+1; 
       lambda_delta_x=lambda*delta_x[j_Wi];  //close and jump then it is... 
-      exp_neg_2_lambda_delta_x=exp(-2*lambda_delta_x); 
+      exp_neg_2_lambda_delta_x=exp(-2.0*lambda_delta_x); 
       
       d(0,0)=(exp_neg_2_lambda_delta_x*(3+6*lambda_delta_x+6*pow(lambda_delta_x,2.0)+4*pow(lambda_delta_x,3.0)+2*pow(lambda_delta_x,4.0))-3 )/(-4*pow(lambda,5.0)); 
       d(1, 0)=  d(0, 1)=exp_neg_2_lambda_delta_x*pow(delta_x[j_Wi],4.0)/2.0; 
@@ -136,8 +174,36 @@ List Construct_W_matern_5_2(double sigma2,Eigen::VectorXd delta_x, double lambda
   return Wi; 
 }
 
+////Construct_W_matern_3_2  
+// [[Rcpp::export]] 
+List Construct_W_matern_3_2(double sigma2,Eigen::VectorXd delta_x, double lambda, MatrixXd W0){  //be careful about delta_x,lambda if one only wants to sample one 
+  int num_obs=delta_x.size()+1; 
+  //int num_dim=sigma2.size();  
+  List Wi(num_obs);  
+  Wi[0]=W0; 
+  Eigen::MatrixXd d= Eigen::MatrixXd::Zero(2,2); //the first row has all zeros  
+  
+  double  lambda_delta_x;
+  double  lambda_delta_x_2;
+  double exp_neg_2_lambda_delta_x;
+  int  j_Wi_1;
+  for(int j_Wi=0;j_Wi<(num_obs-1);j_Wi++){ 
+    j_Wi_1= j_Wi+1; 
+    lambda_delta_x=lambda*delta_x[j_Wi];  //close and jump then it is... 
+    exp_neg_2_lambda_delta_x=exp(-2.0*lambda_delta_x); 
+    lambda_delta_x_2=pow(lambda_delta_x,2.0);
+    d(0,0)=1-(pow(lambda_delta_x,2.0)+pow(1+lambda_delta_x,2.0))*exp_neg_2_lambda_delta_x;
+    d(0,1)=d(1,0)=2.0*lambda*pow(lambda_delta_x,2.0)*exp_neg_2_lambda_delta_x;
+    d(1,1)=pow(lambda,2.0)*(1-(lambda_delta_x_2+pow(1-lambda_delta_x,2.0))*exp_neg_2_lambda_delta_x);
 
-////Construct_W_matern_5_2  
+    d=d*sigma2; 
+    Wi[j_Wi_1]=d; 
+  } 
+  return Wi; 
+}
+
+
+////Construct_W_exp
 // [[Rcpp::export]] 
 List Construct_W_exp(double sigma2, Eigen::VectorXd delta_x, double lambda, MatrixXd W0){  //be careful about delta_x,lambda if one only wants to sample one 
   int num_obs=delta_x.size()+1; 
@@ -225,6 +291,13 @@ List Get_log_det_S2(const Eigen::VectorXd param,const bool have_noise,const Eige
   GG=Construct_G_matern_5_2(delta_x,lambda);  
   W=Construct_W_matern_5_2(1.0,delta_x,lambda,W0);
   
+  }else if(kernel_type=="matern_3_2"){
+    lambda=sqrt(3.0)/gamma;
+    
+    W0=Construct_W0_matern_3_2(1.0,lambda);   
+    GG=Construct_G_matern_3_2(delta_x,lambda);  
+    W=Construct_W_matern_3_2(1.0,delta_x,lambda,W0);
+    
   }else if(kernel_type=="exp"){
     
     lambda=1.0/gamma;
@@ -542,6 +615,13 @@ List Kalman_smoother(const VectorXd param,const bool have_noise,const VectorXi i
     GG=Construct_G_matern_5_2(delta_x_all,lambda);  
     W=Construct_W_matern_5_2(1.0,delta_x_all,lambda,W0);
     
+  }else if(kernel_type=="matern_3_2"){
+    lambda=sqrt(3.0)/gamma;
+    
+    W0=Construct_W0_matern_3_2(1.0,lambda);   
+    GG=Construct_G_matern_3_2(delta_x_all,lambda);  
+    W=Construct_W_matern_3_2(1.0,delta_x_all,lambda,W0);
+    
   }else if(kernel_type=="exp"){
     
     lambda=1.0/gamma;
@@ -612,7 +692,15 @@ MatrixXd  Sample_KF(const List GG,const List  W,const Eigen::MatrixXd C0,const d
            random_norm_theta(t,i)= R::rnorm(0,1.0);
         }
       }
-
+  }else if(kernel_type=="matern_3_2"){
+    theta_sample=Eigen::MatrixXd::Zero(2,n);
+    random_norm_theta=Eigen::MatrixXd::Zero(2,n+1);
+    
+    for(int t=0;t<2;t++){
+      for(int i=0;i<(n+1);i++){
+        random_norm_theta(t,i)= R::rnorm(0,1.0);
+      }
+    }
   }else if(kernel_type=="exp"){
       theta_sample=Eigen::MatrixXd::Zero(1,n);
     
@@ -681,6 +769,16 @@ MatrixXd  Sample_KF_post(const VectorXi index_obs,const List C_R_K_Q, const Eige
         random_norm_theta(t,i)= R::rnorm(0,1.0);
       }
     }
+  }else if(kernel_type=="matern_3_2"){
+    theta_sample=Eigen::MatrixXd::Zero(2,n);
+    random_norm_theta=Eigen::MatrixXd::Zero(2,n+1);
+    
+    for(int t=0;t<2;t++){
+      for(int i=0;i<(n+1);i++){
+        random_norm_theta(t,i)= R::rnorm(0,1.0);
+      }
+    }
+    
   }else if(kernel_type=="exp"){
     theta_sample=Eigen::MatrixXd::Zero(1,n);
     
@@ -967,15 +1065,15 @@ List Get_G_log_det_cov(const Eigen::VectorXd param,const Eigen::MatrixXd output,
       // return_list.push_back( ((z.transpose()*z).array()).matrix());
       
     }
-  }else if(kernel_type=="exp"){
-    lambda=(1.0/gamma.array()).matrix();
+  }else if(kernel_type=="matern_3_2"){
+    lambda=(sqrt(3.0)/gamma.array()).matrix();
     
     for(int i=0;i<d;i++){  
-      W0=Construct_W0_exp(1.0,lambda[i]);  
-      GG=Construct_G_exp(delta_x,lambda[i]);  
-      W=Construct_W_exp(1.0,delta_x,lambda[i],W0);
+      W0=Construct_W0_matern_3_2(1.0,lambda[i]);  
       
+      GG=Construct_G_matern_3_2(delta_x,lambda[i]);  
       
+      W=Construct_W_matern_3_2(1.0,delta_x,lambda[i],W0);
       
       Q_K=Get_Q_K(GG,W,W0,VV[i]);
       
@@ -986,8 +1084,27 @@ List Get_G_log_det_cov(const Eigen::VectorXd param,const Eigen::MatrixXd output,
       z=Get_Y_minus_a_1_scaled_matrix_2d(output_t,GG,Q_K[0],Q_K[1]);
       
       return_list.push_back( (1.0/tau[i]*(z.transpose()*z).array()).matrix());
-      // return_list.push_back( ((z.transpose()*z).array()).matrix());
+
+    }
+    
+  }else if(kernel_type=="exp"){
+    lambda=(1.0/gamma.array()).matrix();
+    
+    for(int i=0;i<d;i++){  
+      W0=Construct_W0_exp(1.0,lambda[i]);  
+      GG=Construct_G_exp(delta_x,lambda[i]);  
+      W=Construct_W_exp(1.0,delta_x,lambda[i],W0);
       
+      Q_K=Get_Q_K(GG,W,W0,VV[i]);
+      
+      Q=Q_K[0];
+      
+      log_det_cov[i]=(tau[i]*Q.array()).log().sum();
+      
+      z=Get_Y_minus_a_1_scaled_matrix_2d(output_t,GG,Q_K[0],Q_K[1]);
+      
+      return_list.push_back( (1.0/tau[i]*(z.transpose()*z).array()).matrix());
+
     }
     
   }
@@ -1111,7 +1228,7 @@ List fmou_cpp(const Eigen::MatrixXd & output, int d,
   Eigen::MatrixXd output_2 = output * output.transpose();
   std::string kernel_type = "exp";
   Eigen::VectorXd delta_x = Eigen::VectorXd::Ones(output.cols() - 1);
-  double trace_output_2 = output.array().square().sum();
+  //double trace_output_2 = output.array().square().sum();//unused
   
   
   // Initialization
@@ -1475,7 +1592,7 @@ Eigen::MatrixXd Optimization_Stiefel_Manifold(const Eigen::MatrixXd A_ini, const
   Eigen::MatrixXd gradient_F_Y_tau=B-A_cur*B.transpose()*A_cur;
   
   double F_Y_tau=F_Funct(A_cur,G);
-  double F_Y_0;
+  //double F_Y_0;
   
   bool find_tau;
   Eigen::MatrixXd gradient_F_A_cur;
@@ -1538,7 +1655,7 @@ Eigen::MatrixXd Optimization_Stiefel_Manifold(const Eigen::MatrixXd A_ini, const
       
       F_Y_tau=F_Funct(Y_tau,G);
       
-      F_Y_0=F_Funct(A_cur,G);
+      //F_Y_0=F_Funct(A_cur,G); //unused
       
       B_A=F_Funct_Dev(A_cur,G);
       
@@ -1735,6 +1852,15 @@ List IKF_CG_particle(VectorXd param, const  String kernel_type, const VectorXd d
     GG=Construct_G_matern_5_2(delta_x_all,lambda);  
     W=Construct_W_matern_5_2(1.0,delta_x_all,lambda,W0);
     
+  }else if(kernel_type=="matern_3_2"){
+    lambda=sqrt(3.0)/gamma;
+    
+    //param.tail(k).array().exp().matrix();
+    
+    W0=Construct_W0_matern_3_2(1.0,lambda);   
+    GG=Construct_G_matern_3_2(delta_x_all,lambda);  
+    W=Construct_W_matern_3_2(1.0,delta_x_all,lambda,W0);
+    
   }else if(kernel_type=="exp"){
     
     lambda=1.0/gamma;
@@ -2128,6 +2254,15 @@ List IKF_CG_particle_cell(VectorXd param, const  String kernel_type, const Vecto
     GG=Construct_G_matern_5_2(delta_x_all,lambda);  
     W=Construct_W_matern_5_2(1.0,delta_x_all,lambda,W0);
     
+  }else if(kernel_type=="matern_3_2"){
+    lambda=sqrt(3.0)/gamma;
+    
+    //param.tail(k).array().exp().matrix();
+    
+    W0=Construct_W0_matern_3_2(1.0,lambda);   
+    GG=Construct_G_matern_3_2(delta_x_all,lambda);  
+    W=Construct_W_matern_3_2(1.0,delta_x_all,lambda,W0);
+    
   }else if(kernel_type=="exp"){
     
     lambda=1.0/gamma;
